import django_tables2 as tables

from netbox.tables import NetBoxTable, ChoiceFieldColumn, columns

from . import models
from . import columns as riskColumns

# ThreatSource Tables


class ThreatSourceTable(NetBoxTable):
    
    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = models.ThreatSource
        fields = ["name", "threat_type", "capability", "intent", "targeting"]


# ThreatEvent Tables


class ThreatEventTable(NetBoxTable):
    
    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = models.ThreatEvent
        fields = ["name", "threat_source", "relevance", "likelihood", "impact"]


# Vulnerability Tables


class VulnerabilityTable(NetBoxTable):

    name = tables.Column(linkify=True)
    
    class Meta(NetBoxTable.Meta):
        model = models.Vulnerability
        fields = ["name", "cve", "description", "asset"]


# VulnerabilityAssignment Tables


class VulnerabilityAssignmentTable(NetBoxTable):

    actions = columns.ActionsColumn(actions=('delete',))

    class Meta(NetBoxTable.Meta):
        model = models.VulnerabilityAssignment
        fields = ["vulnerability", "vulnerability__cve", "vulnerability__description"]

class VulnerabilityAssignmentListTable(NetBoxTable):

    actions = columns.ActionsColumn(actions=('delete',))

    asset = tables.Column(linkify=True)

    asset_object_type = tables.Column(verbose_name="Asset Type")

    class Meta(NetBoxTable.Meta):
        model = models.VulnerabilityAssignment
        fields = ["asset", "asset_object_type"]

class VulnerabilityExploitListTable(NetBoxTable):

    actions = columns.ActionsColumn(actions=('delete',))

    asset = tables.Column(linkify=True)

    vulnerability = tables.Column(linkify=True)

    asset_object_type = tables.Column(verbose_name="Asset Type")

    class Meta(NetBoxTable.Meta):
        model = models.VulnerabilityAssignment
        fields = ["asset", "vulnerability", "asset_object_type"] 


# Risk Tables

class RiskTable(NetBoxTable):

    name = tables.Column(linkify=True)

    risk_level = tables.Column(verbose_name="Risk Level")

    class Meta(NetBoxTable.Meta):
        model = models.Risk
        fields = ["name", "threat_event", "likelihood", "impact", "risk_level"]

# CVE Tables

class CveTable(tables.Table):

    id = tables.Column(attrs={'td': {'class': 'text-end text-nowrap'}})
    description = tables.Column()

    create = riskColumns.CreateColumn(empty_values=())

    
    class Meta:
        attrs = {
            'class': 'table table-hover object-list',
        }

    


from netbox.filtersets import NetBoxModelFilterSet
from . import models

# ThreatSource Filters


class ThreatSourceFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = models.ThreatSource
        fields = ["threat_type", "capability", "intent", "targeting"]


# ThreatEvent Filters


class ThreatEventFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = models.ThreatEvent
        fields = ["threat_source", "relevance", "likelihood", "impact"]


# Vulnerability Filters


class VulnerabilityFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = models.Vulnerability
        fields = ["name", "cve"]


# VulnerabilityAssignment Filters


class VulnerabilityAssignmentFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = models.VulnerabilityAssignment
        fields = ["vulnerability"]

# Risk Filters

class RiskFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = models.Risk
        fields = ["name", "threat_event", "description", "impact", "likelihood"]



from netbox.views import generic
from dcim.models import Device, Site
from tenancy.models import Tenant
from virtualization.models import VirtualMachine

from utilities.views import ViewTab, register_model_view
from django.contrib.contenttypes.models import ContentType
from django.shortcuts import get_object_or_404

import logging

logger = logging.getLogger(__name__)

from . import forms, models, tables, filters

# ThreatSource Views


class ThreatSourceView(generic.ObjectView):
    queryset = models.ThreatSource.objects.all()


class ThreatSourceListView(generic.ObjectListView):
    queryset = models.ThreatSource.objects.all()
    table = tables.ThreatSourceTable
    filterset = filters.ThreatSourceFilterSet
    filterset_form = forms.ThreatSourceFilterForm


class ThreatSourceEditView(generic.ObjectEditView):
    queryset = models.ThreatSource.objects.all()
    form = forms.ThreatSourceForm


class ThreatSourceDeleteView(generic.ObjectDeleteView):
    queryset = models.ThreatSource.objects.all()


# ThreatEvent Views


class ThreatEventView(generic.ObjectView):
    queryset = models.ThreatEvent.objects.all()

class ThreatEventVulnerabilityView(generic.ObjectView):
    queryset = models.ThreatEvent.objects.all()
    template_name = "nb_risk/threatevent_vulnerabilities.html"

    def get_extra_context(self, request, instance):
        vulnerabilities = instance.vulnerability.all()
        table = tables.VulnerabilityExploitListTable(vulnerabilities)
        data = {
            "tab" : "vulnerabilities",
            "table": table,
        }
        return data

class ThreatEventListView(generic.ObjectListView):
    queryset = models.ThreatEvent.objects.all()
    table = tables.ThreatEventTable
    filterset = filters.ThreatEventFilterSet
    filterset_form = forms.ThreatEventFilterForm


class ThreatEventEditView(generic.ObjectEditView):
    queryset = models.ThreatEvent.objects.all()
    form = forms.ThreatEventForm


class ThreatEventDeleteView(generic.ObjectDeleteView):
    queryset = models.ThreatEvent.objects.all()

class ThreatEventBulkDeleteView(generic.BulkDeleteView):
    queryset = models.ThreatEvent.objects.all()
    table = tables.ThreatEventTable


# Vulnerability Views


class VulnerabilityView(generic.ObjectView):
    queryset = models.Vulnerability.objects.all()

class VulnerabilityAffectedAssetsView(generic.ObjectView):
    queryset = models.Vulnerability.objects.all()
    template_name = "nb_risk/vulnerability_affected_assets.html"

    def get_extra_context(self, request, instance):
        assets = models.VulnerabilityAssignment.objects.filter(vulnerability=instance)
        table = tables.VulnerabilityAssignmentListTable(assets)
        data = {
            "tab" : "affected_assets",
            "table": table,
        }
        return data


class VulnerabilityListView(generic.ObjectListView):
    queryset = models.Vulnerability.objects.all()
    table = tables.VulnerabilityTable
    filterset = filters.VulnerabilityFilterSet
    filterset_form = forms.VulnerabilityFilterForm
    template_name = "nb_risk/vulnerability_list.html"


class VulnerabilityEditView(generic.ObjectEditView):
    queryset = models.Vulnerability.objects.all()
    form = forms.VulnerabilityForm


class VulnerabilityDeleteView(generic.ObjectDeleteView):
    queryset = models.Vulnerability.objects.all()

class VulnerabilityBulkDeleteView(generic.BulkDeleteView):
    queryset = models.Vulnerability.objects.all()
    table = tables.VulnerabilityTable

# Generic Vulnerability List View


def create_view(model, base_template_name):

    name = f"{model._meta.model_name}-vulnerability_list"
    path = f"{model._meta.model_name}-vulnerability_list"

    class View(generic.ObjectChildrenView):
        def __init__(self, *args, **kwargs):
            self.queryset = model.objects.all()
            self.child_model = models.VulnerabilityAssignment
            self.table = tables.VulnerabilityAssignmentTable
            self.template_name = "nb_risk/generic_vulnerability_list.html"
            super().__init__(*args, **kwargs)

        tab = ViewTab(
            label="Vulnerabilities",
            badge=lambda obj: models.VulnerabilityAssignment.objects.filter(
                asset_object_type=ContentType.objects.get_for_model(obj),
                asset_id=obj.id,
            ).count(),
            hide_if_empty=True,
        )

        def get_children(self, request, parent):
            childrens = self.child_model.objects.filter(
                asset_object_type=ContentType.objects.get_for_model(parent),
                asset_id=parent.id,
            )
            return childrens

        def get_extra_context(self, request, instance):
            data = {
                "base_template_name": base_template_name,
            }
            return data

    register_model_view(model, name=name, path=path)(View)


supported_assets = [
    {"model": Device, "template": "dcim/device.html"},
    {"model": VirtualMachine, "template": "virtualization/virtualmachine.html"},
    {"model": Site, "template": "dcim/site.html"},
    {"model": Tenant, "template": "tenancy/tenant.html"},
]
for supported_asset in supported_assets:
    create_view(supported_asset["model"], supported_asset["template"])

# VulnerabilityAssignment Views

class VulnerabilityAssignmentEditView(generic.ObjectEditView):
    queryset = models.VulnerabilityAssignment.objects.all()
    form = forms.VulnerabilityAssignmentForm
    template_name = 'nb_risk/generic_vulnerability_assignment_edit.html'

    def alter_object(self, instance, request, args, kwargs):
        if not instance.pk:
            # Assign the object based on URL kwargs
            content_type = get_object_or_404(ContentType, pk=request.GET.get('asset_object_type'))
            instance.object = get_object_or_404(content_type.model_class(), pk=request.GET.get('asset_id'))
        return instance

    def get_extra_addanother_params(self, request):
        return {
            'asset_object_type': request.GET.get('asset_object_type'),
            'asset_id': request.GET.get('asset_id'),
        }

class VulnerabilityAssignmentDeleteView(generic.ObjectDeleteView):
    queryset = models.VulnerabilityAssignment.objects.all()


# Risk Views

class RiskListView(generic.ObjectListView):
    queryset = models.Risk.objects.all()
    table = tables.RiskTable
    filterset = filters.RiskFilterSet
    filterset_form = forms.RiskFilterForm

class RiskView(generic.ObjectView):
    queryset = models.Risk.objects.all()

class RiskEditView(generic.ObjectEditView):
    queryset = models.Risk.objects.all()
    form = forms.RiskForm

class RiskDeleteView(generic.ObjectDeleteView):
    queryset = models.Risk.objects.all()

class RiskBulkDeleteView(generic.BulkDeleteView):
    queryset = models.Risk.objects.all()
    table = tables.RiskTable


from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.urls import reverse

from netbox.models import NetBoxModel
from . import choices

# ThreatSource Model


class ThreatSource(NetBoxModel):

    name = models.CharField("Name", max_length=100, unique=True)
    threat_type = models.CharField(
        "Threat Type",
        max_length=100,
        choices=choices.ThreatTypeChoices,
        default=choices.ThreatTypeChoices.THREAT_TYPE_1,
    )
    capability = models.CharField(
        "Capability",
        max_length=100,
        choices=choices.CapabilityChoices,
        default=choices.CapabilityChoices.CAPABILITY_1,
    )
    intent = models.CharField("Intent", max_length=100, blank=True)
    targeting = models.CharField("Target", max_length=100, blank=True)
    description = models.CharField("Description", max_length=100, blank=True)
    notes = models.TextField("Notes", blank=True)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse(
            "plugins:nb_risk:threatsource", args=[self.pk]
        )


# Vulnerability Model


class Vulnerability(NetBoxModel):

    name = models.CharField("Name", max_length=100, unique=True)
    cve = models.CharField("CVE", max_length=100, blank=True)
    description = models.CharField("Description", max_length=100, blank=True)
    notes = models.TextField("Notes", blank=True)

    def __str__(self):
        return self.name
    
    def get_absolute_url(self):
        return reverse(
            "plugins:nb_risk:vulnerability", args=[self.pk]
        )

    class Meta:
        verbose_name_plural = "Vulnerabilities"

# VulnearbilityAssingment Model


class VulnerabilityAssignment(NetBoxModel):

    asset_object_type = models.ForeignKey(
        to=ContentType,
        on_delete=models.CASCADE,
        related_name="+",
        blank=True,
        null=True,
    )
    asset_id = models.PositiveIntegerField(blank=True, null=True)
    asset = GenericForeignKey(
        ct_field="asset_object_type",
        fk_field="asset_id",
    )
    vulnerability = models.ForeignKey(
        to=Vulnerability,
        on_delete=models.PROTECT,
        related_name="vulnerability_assignments",
        verbose_name="Vulnerability",
    )

    @property
    def name(self):
        return f"{self.asset} - {self.vulnerability.name}"

    def __str__(self):
        return f"{self.asset} - {self.vulnerability.name}"

    class Meta:
        constraints = (
            models.UniqueConstraint(
                fields=('asset_object_type', 'asset_id', 'vulnerability'),
                name='%(app_label)s_%(class)s_unique_object_vulnerability'
            ),
        )


# ThreatEvent Model


class ThreatEvent(NetBoxModel):

    name = models.CharField("Name", max_length=100, unique=True)
    threat_source = models.ForeignKey(
        to=ThreatSource,
        on_delete=models.PROTECT,
        related_name="threat_events",
        verbose_name="Threat Source",
        blank=True,
        null=True,
    )
    description = models.CharField("Description", max_length=100, blank=True)
    notes = models.TextField("Notes", blank=True)
    relevance = models.CharField(
        "Relevance",
        max_length=100,
        choices=choices.RelevanceChoices,
        default=choices.RelevanceChoices.RELEVANCE_6,
    )
    likelihood = models.CharField(
        "Likelihood",
        max_length=100,
        choices=choices.LikelihoodChoices,
        default=choices.LikelihoodChoices.LIKELIHOOD_1,
    )
    impact = models.CharField("Impact", max_length=100, unique=True)
    
    vulnerability = models.ManyToManyField(
        to=VulnerabilityAssignment,
        related_name="threat_events",
        blank=True,
    )

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse(
            "plugins:nb_risk:threatevent", args=[self.pk]
        )

# Risk Model

class Risk(NetBoxModel):

    name = models.CharField("Name", max_length=100, unique=True)
    notes = models.TextField("Notes", blank=True)
    threat_event = models.ForeignKey(
        to=ThreatEvent,
        on_delete=models.PROTECT,
        related_name="risks",
        verbose_name="Threat Event",
    )
    description = models.CharField("Description", max_length=100, blank=True)
    likelihood = models.CharField(
        "Likelihood",
        max_length=100,
        choices=choices.LikelihoodChoices,
        default=choices.LikelihoodChoices.LIKELIHOOD_1,
    )
    impact = models.CharField(
        "Level of Impact",
        max_length=100,
        choices=choices.ImpactChoices,
        default=choices.ImpactChoices.IMPACT_1,
    )

    @property
    def risk_level(self):
        if self.impact == choices.ImpactChoices.IMPACT_5:
            return choices.RiskLevelChoices.RISK_LEVEL_5
        elif self.impact == choices.ImpactChoices.IMPACT_4:
            if self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_5:
                return choices.RiskLevelChoices.RISK_LEVEL_5
            else:
                return choices.RiskLevelChoices.RISK_LEVEL_4
        elif self.impact == choices.ImpactChoices.IMPACT_3:
            if self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_5:
                return choices.RiskLevelChoices.RISK_LEVEL_5
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_4:
                return choices.RiskLevelChoices.RISK_LEVEL_4
            else:
                return choices.RiskLevelChoices.RISK_LEVEL_3
        elif self.impact == choices.ImpactChoices.IMPACT_2:
            if self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_5:
                return choices.RiskLevelChoices.RISK_LEVEL_4
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_4:
                return choices.RiskLevelChoices.RISK_LEVEL_4
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_3:
                return choices.RiskLevelChoices.RISK_LEVEL_3
            else:
                return choices.RiskLevelChoices.RISK_LEVEL_2
        elif self.impact == choices.ImpactChoices.IMPACT_1:
            if self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_5:
                return choices.RiskLevelChoices.RISK_LEVEL_4
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_4:
                return choices.RiskLevelChoices.RISK_LEVEL_3
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_3:
                return choices.RiskLevelChoices.RISK_LEVEL_2
            elif self.likelihood == choices.LikelihoodChoices.LIKELIHOOD_2:
                return choices.RiskLevelChoices.RISK_LEVEL_1
            else:
                return choices.RiskLevelChoices.RISK_LEVEL_1
                
    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse(
            "plugins:nb_risk:risk", args=[self.pk]
        )


import django_tables2 as tables
from django.urls import reverse
from django.utils.safestring import mark_safe
from django.utils.http import urlencode


class CreateColumn(tables.Column):
    attrs = {'td': {'class': 'text-end text-nowrap'}}

    def render(self, record):
        cve = record['id']
        description = record['description']
        print(cve)
        url = reverse('plugins:nb_risk:vulnerability_add')
        query = {
            'cve': cve,
            'notes': description,
            'name': cve
        }
        encoded_query = urlencode(query)
        url = f'{url}?{encoded_query}'


        html = f'<a href={url} class="btn btn-primary btn-sm"><i class="mdi mdi-plus" aria-hidden="true"></i></a>'
        return mark_safe(html)
        


from extras.plugins import PluginTemplateExtension
from django.conf import settings
from packaging import version

NETBOX_CURRENT_VERSION = version.parse(settings.VERSION)

def create_button(model_name):
    
    class Button(PluginTemplateExtension):
        model = model_name
        def buttons(self):
            return self.render('nb_risk/vulnerability_assignment_button.html')
    
    return Button

supported_assets = [ "dcim.device", "virtualization.virtualmachine", "dcim.site", "tenancy.tenant" ]
template_extensions = []
for supported_asset in supported_assets:
    template_extensions.append(create_button(supported_asset))





from django.urls import path
from netbox.views.generic import ObjectChangeLogView, ObjectJournalView

from . import models, views, cve

urlpatterns  = (

    # ThreatSource URLs

    path('threat-source/', views.ThreatSourceListView.as_view(), name='threatsource_list'),
    path('threat-source/add/', views.ThreatSourceEditView.as_view(), name='threatsource_add'),
    path('threat-source/<int:pk>/', views.ThreatSourceView.as_view(), name='threatsource'),
    path('threat-source/<int:pk>/edit/', views.ThreatSourceEditView.as_view(), name='threatsource_edit'),
    path('threat-source/<int:pk>/delete/', views.ThreatSourceDeleteView.as_view(), name='threatsource_delete'),
    path('threat-source/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='threatsource_changelog', kwargs={
        'model': models.ThreatSource
    }),

    # ThreatEvent URLs

    path('threat-event/', views.ThreatEventListView.as_view(), name='threatevent_list'),
    path('threat-event/add/', views.ThreatEventEditView.as_view(), name='threatevent_add'),
    path('threat-event/<int:pk>/', views.ThreatEventView.as_view(), name='threatevent'),
    path('threat-event/<int:pk>/vulnerabilities/', views.ThreatEventVulnerabilityView.as_view(), name='threatevent_vulnerabilities'),
    path('threat-event/<int:pk>/edit/', views.ThreatEventEditView.as_view(), name='threatevent_edit'),
    path('threat-event/<int:pk>/delete/', views.ThreatEventDeleteView.as_view(), name='threatevent_delete'),
    path('threat-event/delete/', views.ThreatEventBulkDeleteView.as_view(), name='threatevent_bulk_delete'),
    path('threat-event/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='threatevent_changelog', kwargs={
        'model': models.ThreatEvent
    }),

    # Vulnerability URLs

    path('vulnerability/', views.VulnerabilityListView.as_view(), name='vulnerability_list'),
    path('vulnerability/add/', views.VulnerabilityEditView.as_view(), name='vulnerability_add'),
    path('vulnerability/<int:pk>/', views.VulnerabilityView.as_view(), name='vulnerability'),
    path('vulnerability/<int:pk>/edit/', views.VulnerabilityEditView.as_view(), name='vulnerability_edit'),
    path('vulnerability/<int:pk>/delete/', views.VulnerabilityDeleteView.as_view(), name='vulnerability_delete'),
    path('vulnerability/delete/', views.VulnerabilityBulkDeleteView.as_view(), name='vulnerability_bulk_delete'),
    path('vulnerability/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='vulnerability_changelog', kwargs={
        'model': models.Vulnerability
    }),
    path('vulnerability/<int:pk>/affected-assets/', views.VulnerabilityAffectedAssetsView.as_view(), name='vulnerability_affected_assets'),
    path('vulnerability/search/', cve.VulnerabilitySearchView.as_view(), name='vulnerability_search'),
    # VulnerabilityAssignment URLs
    path('vulnerability-assignments/add/', views.VulnerabilityAssignmentEditView.as_view(), name='vulnerabilityassignment_add'),
    path('vulnerability-assignments/<int:pk>/delete/', views.VulnerabilityAssignmentDeleteView.as_view(), name='vulnerabilityassignment_delete'),

    # Risk URLs
    path('risk/', views.RiskListView.as_view(), name='risk_list'),
    path('risk/add/', views.RiskEditView.as_view(), name='risk_add'),
    path('risk/<int:pk>/', views.RiskView.as_view(), name='risk'),
    path('risk/<int:pk>/edit/', views.RiskEditView.as_view(), name='risk_edit'),
    path('risk/<int:pk>/delete/', views.RiskDeleteView.as_view(), name='risk_delete'),
    path('risk/delete/', views.RiskBulkDeleteView.as_view(), name='risk_bulk_delete'),
    path('risk/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='risk_changelog', kwargs={
        'model': models.Risk
    }),
    path('risk/<int:pk>/journal/', ObjectJournalView.as_view(), name='risk_journal', kwargs={
        'model': models.Risk
    }),


    
)